{% extends "base.html" %}{% load i18n %}
{% block title %}{% trans "Daily View" %}{% endblock %}
{% block document %}
    <h3>{% trans "Daily View" %}</h3>
    <h4>
        <a href="{% url agenda-daily-view prev_day.year prev_day.month prev_day.day %}">&larr;</a>
        {{ day|date:"DATE_FORMAT" }}
        <a href="{% url agenda-daily-view next_day.year next_day.month next_day.day %}">&rarr;</a>
    </h4>
    <table class="calendar">
        <thead>
            <tr>
                <th>{% trans "Time" %}</th>
                <th>{% trans "Main Room" %}</th>
                <th>{% trans "Private 2" %}</th>
                <th>{% trans "Private 3" %}</th>
                <th colspan="4">{% trans "Private 4" %}</th>
            </tr>
        </thead>
        <tbody>
            {% for tm,cells in timeslots %}
            <tr>
                <th><a href="{% url agenda-add-event %}?dtstart={{ tm.isoformat }}">{{ tm|date:"H:i" }}</a></th>
                {% for cell in cells %}
                <td{% if cell.event_class %} class="{{cell.event_class}}"{% endif %}>{{ cell }}</td>    
                {% endfor %}
            </tr>
            {% endfor %}
        </tbody>
    </table>
{% endblock %}