{% load i18n %}
        <tr><th>{{ recurrence_form.day.label_tag}}</th>
            <td>{{ recurrence_form.day }}</td></tr>
        <tr><th>{{ recurrence_form.start_time_delta.label_tag }}</th>
            <td>{{ recurrence_form.start_time_delta }}</td></tr>
        <tr><th>{{ recurrence_form.end_time_delta.label_tag }}</th>
            <td>{{ recurrence_form.end_time_delta }}</td></tr>

        <tr><th>{{ recurrence_form.repeats.label_tag }}</th>
            <td>
                {{ recurrence_form.count.errors }}
                <ul>
                    <li>
                        <input checked="checked" type="radio" id="id_repeats_0" 
                            value="count" name="repeats" />{% blocktrans with recurrence_form.count as times %}<label for="id_repeats_0"> A total of</label>
                        {{ times }} time(s){% endblocktrans %}</li>
                    <li>
                        <input type="radio" id="id_repeats_1" value="until" 
                            name="repeats" /><label for="id_repeats_1"> {% trans "Until" %} </label>
                        {{ recurrence_form.until }}</li>
                </ul></td></tr>

        <tr><th>{{ recurrence_form.freq.label_tag }}</th>
            <td>
                {{ recurrence_form.interval.errors }}
                <div style="float: left">{% blocktrans with recurrence_form.interval as interval %}Every {{ interval }} {% endblocktrans %}</div>
                <ul id="recurrence-options">
                    <li><label for="id_freq_0">
                        <input type="radio" id="id_freq_0" 
                            value="3" name="freq" /> {% trans "Day(s)" %}</label>
                    </li>
                    <li>
                        <label for="id_freq_1">
                            <input checked="checked" type="radio" id="id_freq_1" 
                                value="2" name="freq" /> {% trans "Week(s)" %}</label>
                        <div class="horizontal form-indent">
                            {{ recurrence_form.week_days }}
                        </div>
                        <br clear="all" />
                    </li>
                    <li>
                        <label for="id_freq_2">
                            <input type="radio" id="id_freq_2" 
                                value="1" name="freq" /> {% trans "Month(s)" %}</label>
                        <div class="form-indent">
                            <ul>
                                <li>
                                    <label for="id_month_option_0">
                                        <input type="radio" id="id_month_option_0" 
                                            value="on" name="month_option" /> 
                                        {% trans "On the" %}</label>
                                    {{ recurrence_form.month_ordinal }}
                                    {{ recurrence_form.month_ordinal_day }}
                                </li>
                                <li>
                                    <label for="id_month_option_1">
                                        <input checked="checked" type="radio" 
                                            id="id_month_option_1" value="each" 
                                            name="month_option" /> {% trans "Each" %}:</label>
                                    <br clear="all" />
                                    <div class="horizontal form-indent">
                                        {{ recurrence_form.each_month_day }}
                                    </div>
                                    <br clear="all" />
                                </li>
                            </ul>
                        </div>
                    </li>
                    <li>
                        <label for="id_freq_3">
                            <input type="radio" id="id_freq_3" 
                                value="0" name="freq" /> {% trans "Year(s)" %}</label>
                        <div class="form-indent">
                            <div class="horizontal">
                                {{ recurrence_form.year_months}} 
                            </div>
                            <br clear="all" />
                            <div class="form-indent">
                            <label for="id_is_year_month_ordinal">
                                {{ recurrence_form.is_year_month_ordinal }} 
                                {% trans "On the" %} </label>
                            
                            {{ recurrence_form.year_month_ordinal }}
                            {{ recurrence_form.year_month_ordinal_day }}
                            </div>
                        </div>
                    </li>
                </ul>
            </td>
        </tr>


