{% extends "base.html" %}{% load i18n %}{% csrf_token %}
{% block title %}{% trans "Event Occurrence" %}{% endblock %}
{% block document %}
     <h3>{% trans "Event Occurrence" %}</h3>
     <h4>
         <a href="{{ occurrence.event.get_absolute_url }}">{{ occurrence.title }}</a>
         &ndash;
         {% with occurrence.start_time as st  %}
         <a href="{% url agenda-daily-view st.year st.month st.day %}">
            {{ st|date:"DATETIME_FORMAT" }}</a>
        </h4>
        {% endwith %}
     <dl>
         <dt>{% trans "Event type" %}:</dt>
         <dd>{{ occurrence.event.event_type }}</dd>
         
         <dt>{% trans "Description" %}:</dt>
         <dd>{{ occurrence.event.description|default:"None" }}</dd>
         
         <dt>{% trans "Note(s)" %}:</dt>
         <dd>
             {% with occurrence.event.notes.all as notes %}
             {% if notes %}
             <ul>
                 {% for note in notes %}
                 <li>{{ note }}</li>
                 {% endfor %}
             </ul>
             {% else %}None
             {% endif %}
             {% endwith %}
         </dd>
     </dl>
     
     <form action="" method="post">
     <table>
         <tfoot>
             <tr>
                 <td colspan="2"><input type="submit" value="{% trans "Update" %}" /></td>
             </tr>
         </tfoot>
         <tbody>
             {{ form }}
         </tbody>
     </table>
     </form>

{% endblock %}